"""Dashboard view modules.

Each view module provides a `render()` function that renders the page content.
"""

__all__ = [
    "overview",
    "errors",
    "metrics",
    "providers",
    "sdd_workflow",
]
